@@ -27,6 +27,8 @@
 #include "pygobject-private.h"
 
 static PyObject *gerror_exc = NULL;
+static const gchar *pyginterface_type_id   = "PyGInterface::type";
+GQuark pyginterface_type_key  = 0;
 
 static void pyg_flags_add_constants(PyObject *module, GType flags_type,
 				    const gchar *strip_prefix);
@@ -234,7 +236,7 @@ pyg_interface_init(PyObject *self, PyObject *args, PyObject *kwargs)
     return -1;
 }
 
-static PyTypeObject PyGInterface_Type = {
+PyTypeObject PyGInterface_Type = {
     PyObject_HEAD_INIT(NULL)
     0,                                  /* ob_size */
     "gobject.GInterface",               /* tp_name */
@@ -275,7 +277,7 @@ static PyTypeObject PyGInterface_Type = {
     (initproc)pyg_interface_init,	/* tp_init */
     (allocfunc)0,			/* tp_alloc */
     (newfunc)0,				/* tp_new */
-    object_free,			/* tp_free */
+    (freefunc)object_free,		/* tp_free */
     (inquiry)0,				/* tp_is_gc */
     (PyObject *)0,			/* tp_bases */
 };
@@ -311,7 +313,10 @@ pyg_register_interface(PyObject *dict, const gchar *class_name,
         Py_DECREF(o);
     }
 
+    g_type_set_qdata(gtype, pyginterface_type_key, type);
+    
     PyDict_SetItemString(dict, (char *)class_name, (PyObject *)type);
+    
 }
 
 /* -------------- GMainContext objects ---------------------------- */
@@ -1900,6 +1905,42 @@ pyg_error_check(GError **error)
     return FALSE;
 }
 
+
+static PyObject *
+_pyg_strv_from_gvalue(const GValue *value)
+{
+    gchar    **argv = (gchar **) g_value_get_boxed(value);
+    int        argc, i;
+    PyObject  *py_argv;
+
+    for (argc = -1; argv[++argc];);
+    py_argv = PyList_New(argc);
+    for (i = 0; i < argc; ++i)
+	PyList_SET_ITEM(py_argv, i, PyString_FromString(argv[i]));
+    return py_argv;
+}
+
+static int
+_pyg_strv_to_gvalue(GValue *value, PyObject *obj)
+{
+    int     argc, i;
+    gchar **argv;
+
+    if (!PySequence_Check(obj)) return -1;
+    argc = PySequence_Length(obj);
+    for (i = 0; i < argc; ++i)
+	if (!PyString_Check(PySequence_Fast_GET_ITEM(obj, i)))
+	    return -1;
+    argv = g_new(gchar *, argc + 1);
+    for (i = 0; i < argc; ++i)
+	argv[i] = g_strdup(PyString_AsString(PySequence_Fast_GET_ITEM(obj, i)));
+    argv[i] = NULL;
+    g_value_init(value, G_TYPE_STRV);
+    g_value_take_boxed(value, argv);
+    return 0;
+}
+
+
 /* ----------------- gobject module initialisation -------------- */
 
 struct _PyGObject_Functions pygobject_api_functions = {
@@ -1991,6 +2032,7 @@ initgobject(void)
 			 pyg_object_descr_doc_get());
     PyDict_SetItemString(PyGInterface_Type.tp_dict, "__gdoc__",
 			 pyg_object_descr_doc_get());
+    pyginterface_type_key = g_quark_from_static_string(pyginterface_type_id);
 
     PyGBoxed_Type.ob_type = &PyType_Type;
     PyGBoxed_Type.tp_alloc = PyType_GenericAlloc;
@@ -2087,4 +2129,8 @@ initgobject(void)
     PyModule_AddObject(m, "TYPE_PARAM", pyg_type_wrapper_new(G_TYPE_PARAM));
     PyModule_AddObject(m, "TYPE_OBJECT", pyg_type_wrapper_new(G_TYPE_OBJECT));
     PyModule_AddObject(m, "TYPE_PYOBJECT", pyg_type_wrapper_new(PY_TYPE_OBJECT));
+
+    pyg_register_boxed_custom(G_TYPE_STRV,
+			      _pyg_strv_from_gvalue,
+			      _pyg_strv_to_gvalue);
 }
