@@ -1,4 +1,24 @@
-/* -*- Mode: C; c-basic-offset: 4 -*- */
+/* -*- Mode: C; c-basic-offset: 4 -*-
+ * pygtk- Python bindings for the GTK toolkit.
+ * Copyright (C) 1998-2003  James Henstridge
+ *
+ *   gtk-types.c: wrappers for some specialised GTK types.
+ *
+ * This library is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU Lesser General Public
+ * License as published by the Free Software Foundation; either
+ * version 2.1 of the License, or (at your option) any later version.
+ *
+ * This library is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * Lesser General Public License for more details.
+ *
+ * You should have received a copy of the GNU Lesser General Public
+ * License along with this library; if not, write to the Free Software
+ * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
+ * USA
+ */
 #include <gtk/gtk.h>
 #include "pygtk-private.h"
 #include <structmember.h>
@@ -906,7 +926,7 @@ pygtk_tree_model_row_get_parent(PyGtkTreeModelRow *self, void *closure)
 {
     GtkTreeIter parent;
 
-    if (gtk_tree_model_iter_parent(self->model, &self->iter, &parent))
+    if (gtk_tree_model_iter_parent(self->model, &parent, &self->iter))
 	return _pygtk_tree_model_row_new(self->model, &parent);
     Py_INCREF(Py_None);
     return Py_None;
